@@ -947,6 +947,24 @@ void QtDocGenerator::writeFormatedText(QTextStream& s, const Documentation& doc,
     s << endl;
 }
 
+static void writeInheritedByList(QTextStream& s, const AbstractMetaClass* metaClass, const AbstractMetaClassList& allClasses)
+{
+    AbstractMetaClassList res;
+    foreach (AbstractMetaClass* c, allClasses) {
+        if (c != metaClass && c->inheritsFrom(metaClass))
+            res << c;
+    }
+
+    if (res.isEmpty())
+        return;
+
+    s << "**Inherited by:** ";
+    QStringList classes;
+    foreach (AbstractMetaClass* c, res)
+        classes << QString(":ref:`%1`").arg(getClassTargetFullName(c, false));
+    s << classes.join(", ") << endl << endl;
+}
+
 void QtDocGenerator::generateClass(QTextStream& s, const AbstractMetaClass* metaClass)
 {
     ReportHandler::debugSparse("Generating Documentation for " + metaClass->fullName());
@@ -966,6 +984,9 @@ void QtDocGenerator::generateClass(QTextStream& s, const AbstractMetaClass* meta
     s << ".. inheritance-diagram:: " << className << endl
       << "    :parts: 2" << endl << endl; // TODO: This would be a parameter in the future...
 
+
+    writeInheritedByList(s, metaClass, classes());
+
     if (metaClass->typeEntry() && (metaClass->typeEntry()->version() != 0))
         s << ".. note:: This class was introduced in Qt " << metaClass->typeEntry()->version() << endl;
 
@@ -1007,6 +1028,9 @@ void QtDocGenerator::generateClass(QTextStream& s, const AbstractMetaClass* meta
 void QtDocGenerator::writeFunctionList(QTextStream& s, const AbstractMetaClass* cppClass)
 {
     QStringList functionList;
+    QStringList virtualList;
+    QStringList signalList;
+    QStringList slotList;
     QStringList staticFunctionList;
 
     foreach (AbstractMetaFunction* func, cppClass->functions()) {
@@ -1020,7 +1044,6 @@ void QtDocGenerator::writeFunctionList(QTextStream& s, const AbstractMetaClass*
             className =  getClassTargetFullName(func->implementingClass()->enclosingClass()) + '.';
         QString funcName = getFuncName(func);
 
-        QStringList& list = func->isStatic() ? staticFunctionList : functionList;
         QString str("def :meth:`");
 
         str += funcName;
@@ -1032,7 +1055,16 @@ void QtDocGenerator::writeFunctionList(QTextStream& s, const AbstractMetaClass*
         str += parseArgDocStyle(cppClass, func);
         str += ')';
 
-        list << str;
+        if (func->isStatic())
+            staticFunctionList << str;
+        else if (func->isVirtual())
+            virtualList << str;
+        else if (func->isSignal())
+            signalList << str;
+        else if (func->isSlot())
+            slotList << str;
+        else
+            functionList << str;
     }
 
     if ((functionList.size() > 0) || (staticFunctionList.size() > 0)) {
@@ -1042,36 +1074,28 @@ void QtDocGenerator::writeFunctionList(QTextStream& s, const AbstractMetaClass*
         s << "Synopsis" << endl
           << "--------" << endl << endl;
 
-        if (functionList.size() > 0) {
-            s << "Functions" << endl
-              << "^^^^^^^^^" << endl << endl;
+        writeFunctionBlock(s, "Functions", functionList);
+        writeFunctionBlock(s, "Virtual functions", virtualList);
+        writeFunctionBlock(s, "Slots", slotList);
+        writeFunctionBlock(s, "Signals", signalList);
+        writeFunctionBlock(s, "Static functions", staticFunctionList);
+    }
+}
 
-            qSort(functionList);
-            foreach (QString func, functionList) {
-                row << func;
-                functionTable << row;
-                row.clear();
-            }
+void QtDocGenerator::writeFunctionBlock(QTextStream& s, const QString& title, QStringList& functions)
+{
+    if (functions.size() > 0) {
+        s << title << endl
+          << QString('^').repeated(title.size()) << endl;
 
-            functionTable.normalize();
-            s << functionTable << endl;
-            functionTable.clear();
-        }
+        qSort(functions);
 
-        if (staticFunctionList.size() > 0) {
-            s << "Static functions" << endl
-              << "^^^^^^^^^^^^^^^^" << endl;
+        s << ".. container:: function_list" << endl << endl;
+        Indentation indentation(INDENT);
+        foreach (QString func, functions)
+            s << '*' << INDENT << func << endl;
 
-            qSort(staticFunctionList);
-            foreach (QString func, staticFunctionList) {
-                row << func;
-                functionTable << row;
-                row.clear();
-            }
-
-            functionTable.normalize();
-            s << functionTable << endl;
-        }
+        s << endl << endl;
     }
 }
 
@@ -1409,6 +1433,64 @@ void QtDocGenerator::writeFunction(QTextStream& s, bool writeDoc, const Abstract
     }
 }
 
+static void writeFancyToc(QTextStream& s, const QStringList& items, int cols = 4)
+{
+    typedef QMap<QChar, QStringList> TocMap;
+    TocMap tocMap;
+    QChar Q('Q');
+    QChar idx;
+    foreach (QString item, items) {
+        if (item.isEmpty())
+            continue;
+        if (item.startsWith(Q) && item.length() > 1)
+            idx = item[1];
+        item.chop(4); // Remove the .rst extension
+        tocMap[idx] << item;
+    }
+    QtXmlToSphinx::Table table;
+    QtXmlToSphinx::TableRow row;
+
+    int itemsPerCol = (items.size() + tocMap.size()*2) / cols;
+    QString currentColData;
+    int i = 0;
+    QTextStream ss(&currentColData);
+    QMutableMapIterator<QChar, QStringList> it(tocMap);
+    while (it.hasNext()) {
+        it.next();
+        qSort(it.value());
+
+        if (i)
+            ss << endl;
+
+        ss << "**" << it.key() << "**" << endl << endl;
+        i += 2; // a letter title is equivalent to two entries in space
+        foreach (QString item, it.value()) {
+            ss << "* :doc:`" << item << "`" << endl;
+            ++i;
+
+            // end of column detected!
+            if (i > itemsPerCol) {
+                ss.flush();
+                QtXmlToSphinx::TableCell cell(currentColData);
+                row << cell;
+                currentColData.clear();
+                i = 0;
+            }
+        }
+    }
+    if (i) {
+        ss.flush();
+        QtXmlToSphinx::TableCell cell(currentColData);
+        row << cell;
+        currentColData.clear();
+        i = 0;
+    }
+    table << row;
+    table.normalize();
+    s << ".. container:: pysidetoc" << endl << endl;
+    s << table;
+}
+
 void QtDocGenerator::finishGeneration()
 {
     if (classes().isEmpty())
@@ -1422,14 +1504,12 @@ void QtDocGenerator::finishGeneration()
 
         s << ".. module:: " << it.key() << endl << endl;
 
-        QString title = it.key() + " contents";
+        QString title = it.key();
         s << title << endl;
         s << createRepeatedChar(title.length(), '*') << endl << endl;
-        s << ".. toctree::" << endl;
 
         /* Avoid showing "Detailed Description for *every* class in toc tree */
         Indentation indentation(INDENT);
-        s << INDENT << ":maxdepth: 1" << endl << endl;
 
         // Search for extra-sections
         if (!m_extraSectionDir.isEmpty()) {
@@ -1450,11 +1530,18 @@ void QtDocGenerator::finishGeneration()
             it.value().append(fileList);
         }
 
-        qSort(it.value());
-        foreach (QString className, it.value()) {
-            s << INDENT << className << endl;
+        writeFancyToc(s, it.value());
+
+        s << INDENT << ".. container:: hide" << endl << endl;
+        {
+            Indentation indentation(INDENT);
+            s << INDENT << ".. toctree::" << endl;
+            Indentation deeperIndentation(INDENT);
+            s << INDENT << ":maxdepth: 1" << endl << endl;
+            foreach (QString className, it.value())
+                s << INDENT << className << endl;
+            s << endl << endl;
         }
-        s << endl << endl;
 
         s << "Detailed Description" << endl;
         s << "--------------------" << endl << endl;
